#!/bin/bash

resource_type=$1
resource_name=$2

# statefulset's status.condition field is not set, so we can't use kubectl wait for it
# we tell readiness of a statefulset by comparing `readyReplicas` and desired `replicas`
wait_for_statefulset_readiness() {
  resource=$1
  ready=$(kubectl get statefulset $resource -o jsonpath={.status.readyReplicas})
  count=$(kubectl get statefulset $resource -o jsonpath={.status.replicas})
  updated=$(kubectl get statefulset $resource -o jsonpath={.status.updatedReplicas})
  ATTEMPTS=0

  until [[ $ready = $count && $count -gt 0 && $updated = $count ]]
  do
    ATTEMPTS=`expr $ATTEMPTS + 1` ;
    echo "Failed attempts: $ATTEMPTS" ;
    sleep 10 ; # 10 seconds intervals between attempts
    ready=$(kubectl get statefulset $resource -o jsonpath={.status.readyReplicas})
    count=$(kubectl get statefulset $resource -o jsonpath={.status.replicas})
    updated=$(kubectl get statefulset $resource -o jsonpath={.status.updatedReplicas})
  done ;

  echo "statefulset.apps/$resource condition met"
}

echo "Start waiting for ${resource_type} ${resource_name}"

case "${resource_type}" in
pod)
  desired_condition="READY"
  kubectl wait --for=condition=${desired_condition} ${resource_type} ${resource_name} --timeout=-1s # negative tmeout means wait for 1 week
  ;;
deployment)
  desired_condition="AVAILABLE"
  kubectl wait --for=condition=${desired_condition} ${resource_type} ${resource_name} --timeout=-1s
  ;;
job)
  desired_condition="COMPLETE"
  kubectl wait --for=condition=${desired_condition} ${resource_type} ${resource_name} --timeout=-1s
  ;;
statefulset)
  wait_for_statefulset_readiness ${resource_name}
  ;;
*)
  printf 'ERROR: Unsupported resource type: %s\n' "$1" >&2
  exit 1
  ;;
esac

echo "Finish waiting for ${resource_type} ${resource_name}"
exit 0
